@@ -60,6 +60,7 @@ struct _GstRTSPMediaFactoryPrivate
   guint buffer_size;
   GstRTSPAddressPool *pool;
   GstRTSPTransportMode transport_mode;
+  gboolean stop_on_disconnect;
 
   GstClockTime rtx_time;
   guint latency;
@@ -68,6 +69,8 @@ struct _GstRTSPMediaFactoryPrivate
   GHashTable *medias;           /* protected by medias_lock */
 
   GType media_gtype;
+
+  GstClock *clock;
 };
 
 #define DEFAULT_LAUNCH          NULL
@@ -80,6 +83,7 @@ struct _GstRTSPMediaFactoryPrivate
 #define DEFAULT_BUFFER_SIZE     0x80000
 #define DEFAULT_LATENCY         200
 #define DEFAULT_TRANSPORT_MODE  GST_RTSP_TRANSPORT_MODE_PLAY
+#define DEFAULT_STOP_ON_DISCONNECT TRUE
 
 enum
 {
@@ -93,6 +97,8 @@ enum
   PROP_BUFFER_SIZE,
   PROP_LATENCY,
   PROP_TRANSPORT_MODE,
+  PROP_STOP_ON_DISCONNECT,
+  PROP_CLOCK,
   PROP_LAST
 };
 
@@ -202,6 +208,19 @@ gst_rtsp_media_factory_class_init (GstRTSPMediaFactoryClass * klass)
           GST_TYPE_RTSP_TRANSPORT_MODE, DEFAULT_TRANSPORT_MODE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_STOP_ON_DISCONNECT,
+      g_param_spec_boolean ("stop-on-disconnect", "Stop On Disconnect",
+          "If media from this factory should be stopped "
+          "when a client disconnects without TEARDOWN",
+          DEFAULT_STOP_ON_DISCONNECT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_CLOCK,
+      g_param_spec_object ("clock", "Clock",
+          "Clock to be used by the pipelines created for all "
+          "medias of this factory", GST_TYPE_CLOCK,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_rtsp_media_factory_signals[SIGNAL_MEDIA_CONSTRUCTED] =
       g_signal_new ("media-constructed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRTSPMediaFactoryClass,
@@ -240,6 +259,7 @@ gst_rtsp_media_factory_init (GstRTSPMediaFactory * factory)
   priv->buffer_size = DEFAULT_BUFFER_SIZE;
   priv->latency = DEFAULT_LATENCY;
   priv->transport_mode = DEFAULT_TRANSPORT_MODE;
+  priv->stop_on_disconnect = DEFAULT_STOP_ON_DISCONNECT;
 
   g_mutex_init (&priv->lock);
   g_mutex_init (&priv->medias_lock);
@@ -304,6 +324,13 @@ gst_rtsp_media_factory_get_property (GObject * object, guint propid,
       g_value_set_flags (value,
           gst_rtsp_media_factory_get_transport_mode (factory));
       break;
+    case PROP_STOP_ON_DISCONNECT:
+      g_value_set_boolean (value,
+          gst_rtsp_media_factory_is_stop_on_disonnect (factory));
+      break;
+    case PROP_CLOCK:
+      g_value_take_object (value, gst_rtsp_media_factory_get_clock (factory));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, propid, pspec);
   }
@@ -347,6 +374,13 @@ gst_rtsp_media_factory_set_property (GObject * object, guint propid,
       gst_rtsp_media_factory_set_transport_mode (factory,
           g_value_get_flags (value));
       break;
+    case PROP_STOP_ON_DISCONNECT:
+      gst_rtsp_media_factory_set_stop_on_disconnect (factory,
+          g_value_get_boolean (value));
+      break;
+    case PROP_CLOCK:
+      gst_rtsp_media_factory_set_clock (factory, g_value_get_object (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, propid, pspec);
   }
@@ -862,6 +896,56 @@ gst_rtsp_media_factory_get_protocols (GstRTSPMediaFactory * factory)
   return res;
 }
 
+/**
+ * gst_rtsp_media_factory_set_stop_on_disconnect:
+ * @factory: a #GstRTSPMediaFactory
+ * @stop_on_disconnect: the new value
+ *
+ * Configure if media created from this factory should be stopped
+ * when a client disconnects without sending TEARDOWN.
+ */
+void
+gst_rtsp_media_factory_set_stop_on_disconnect (GstRTSPMediaFactory * factory,
+    gboolean stop_on_disconnect)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+
+  g_return_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory));
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv->stop_on_disconnect = stop_on_disconnect;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+}
+
+/**
+ * gst_rtsp_media_factory_is_stop_on_disconnect:
+ * @factory: a #GstRTSPMediaFactory
+ *
+ * Get if media created from this factory should be stopped when a client
+ * disconnects without sending TEARDOWN.
+ *
+ * Returns: %TRUE if the media will be stopped when a client disconnects
+ *     without sending TEARDOWN.
+ */
+gboolean
+gst_rtsp_media_factory_is_stop_on_disonnect (GstRTSPMediaFactory * factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  gboolean result;
+
+  g_return_val_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory), TRUE);
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  result = priv->stop_on_disconnect;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return result;
+}
+
 /**
  * gst_rtsp_media_factory_set_retransmission_time:
  * @factory: a #GstRTSPMediaFactory
@@ -1142,6 +1226,55 @@ gst_rtsp_media_factory_get_media_gtype (GstRTSPMediaFactory * factory)
   return ret;
 }
 
+/**
+ * gst_rtsp_media_factory_set_clock:
+ * @factory: a #GstRTSPMediaFactory
+ * @clockd: the clock to be used by the media factory
+ *
+ * Configures a specific clock to be used by the pipelines
+ * of all medias created from this factory.
+ *
+ * Since: 1.8
+ */
+void
+gst_rtsp_media_factory_set_clock (GstRTSPMediaFactory * factory,
+    GstClock * clock)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+
+  g_return_if_fail (GST_IS_CLOCK (clock) || clock == NULL);
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv = factory->priv;
+  priv->clock = clock ? gst_object_ref (clock) : NULL;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+}
+
+/**
+ * gst_rtsp_media_factory_get_clock:
+ * @factory: a #GstRTSPMediaFactory
+ *
+ * Returns the clock that is going to be used by the pipelines
+ * of all medias created from this factory.
+ *
+ * Returns: (transfer full): The GstClock
+ *
+ * Since: 1.8
+ */
+GstClock *
+gst_rtsp_media_factory_get_clock (GstRTSPMediaFactory * factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  GstClock *ret;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv = factory->priv;
+  ret = priv->clock ? gst_object_ref (priv->clock) : NULL;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return ret;
+}
+
 static gchar *
 default_gen_key (GstRTSPMediaFactory * factory, const GstRTSPUrl * url)
 {
@@ -1261,6 +1394,11 @@ default_create_pipeline (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   GstElement *pipeline;
 
   pipeline = gst_pipeline_new ("media-pipeline");
+
+  /* FIXME 2.0: This should be done by the caller, not the vfunc. Every
+   * implementation of the vfunc has to call it otherwise at the end.
+   * Also it does not allow use to add further behaviour here that could
+   * be reused by subclasses that chain up */
   gst_rtsp_media_take_pipeline (media, GST_PIPELINE_CAST (pipeline));
 
   return pipeline;
@@ -1270,7 +1408,7 @@ static void
 default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
 {
   GstRTSPMediaFactoryPrivate *priv = factory->priv;
-  gboolean shared, eos_shutdown;
+  gboolean shared, eos_shutdown, stop_on_disconnect;
   guint size;
   GstRTSPSuspendMode suspend_mode;
   GstRTSPProfile profiles;
@@ -1280,6 +1418,7 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   GstClockTime rtx_time;
   guint latency;
   GstRTSPTransportMode transport_mode;
+  GstClock *clock;
 
   /* configure the sharedness */
   GST_RTSP_MEDIA_FACTORY_LOCK (factory);
@@ -1292,6 +1431,8 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   rtx_time = priv->rtx_time;
   latency = priv->latency;
   transport_mode = priv->transport_mode;
+  stop_on_disconnect = priv->stop_on_disconnect;
+  clock = priv->clock ? gst_object_ref (priv->clock) : NULL;
   GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
 
   gst_rtsp_media_set_suspend_mode (media, suspend_mode);
@@ -1303,6 +1444,12 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   gst_rtsp_media_set_retransmission_time (media, rtx_time);
   gst_rtsp_media_set_latency (media, latency);
   gst_rtsp_media_set_transport_mode (media, transport_mode);
+  gst_rtsp_media_set_stop_on_disconnect (media, stop_on_disconnect);
+
+  if (clock) {
+    gst_rtsp_media_set_clock (media, clock);
+    gst_object_unref (clock);
+  }
 
   if ((pool = gst_rtsp_media_factory_get_address_pool (factory))) {
     gst_rtsp_media_set_address_pool (media, pool);
