@@ -39,7 +39,6 @@
 
 #define TEST_MOUNT_POINT  "/test"
 #define TEST_PROTO        "RTP/AVP"
-#define TEST_PROTO_TCP    "RTP/AVP/TCP"
 #define TEST_ENCODING     "X-GST"
 #define TEST_CLOCK_RATE   "90000"
 
@@ -150,11 +149,12 @@ get_client_ports (GstRTSPRange * range)
 
 /* start the tested rtsp server */
 static void
-start_server (void)
+start_server (gboolean set_shared_factory)
 {
   GstRTSPMountPoints *mounts;
   gchar *service;
   GstRTSPMediaFactory *factory;
+  GstRTSPAddressPool *pool;
 
   mounts = gst_rtsp_server_get_mount_points (server);
 
@@ -165,6 +165,16 @@ start_server (void)
   gst_rtsp_mount_points_add_factory (mounts, TEST_MOUNT_POINT, factory);
   g_object_unref (mounts);
 
+  /* use an address pool for multicast */
+  pool = gst_rtsp_address_pool_new ();
+  gst_rtsp_address_pool_add_range (pool,
+      "224.3.0.0", "224.3.0.10", 5000, 5010, 16);
+  gst_rtsp_address_pool_add_range (pool, GST_RTSP_ADDRESS_POOL_ANY_IPV4,
+      GST_RTSP_ADDRESS_POOL_ANY_IPV4, 6000, 6010, 0);
+  gst_rtsp_media_factory_set_address_pool (factory, pool);
+  gst_rtsp_media_factory_set_shared (factory, set_shared_factory);
+  gst_object_unref (pool);
+
   /* set port to any */
   gst_rtsp_server_set_service (server, "0");
 
@@ -428,9 +438,9 @@ static GstSDPMessage *
 do_describe (GstRTSPConnection * conn, const gchar * mount_point)
 {
   GstSDPMessage *sdp_message;
-  gchar *content_type;
-  gchar *content_base;
-  gchar *body;
+  gchar *content_type = NULL;
+  gchar *content_base = NULL;
+  gchar *body = NULL;
   gchar *address;
   gchar *expected_content_base;
 
@@ -467,13 +477,13 @@ do_describe (GstRTSPConnection * conn, const gchar * mount_point)
  * transport must be freed by the caller. */
 static GstRTSPStatusCode
 do_setup_full (GstRTSPConnection * conn, const gchar * control,
-    gboolean use_tcp_transport, const GstRTSPRange * client_ports,
+    GstRTSPLowerTrans lower_transport, const GstRTSPRange * client_ports,
     const gchar * require, gchar ** session, GstRTSPTransport ** transport,
     gchar ** unsupported)
 {
   GstRTSPStatusCode code;
   gchar *session_in = NULL;
-  gchar *transport_string_in = NULL;
+  GString *transport_string_in = NULL;
   gchar **session_out = NULL;
   gchar *transport_string_out = NULL;
 
@@ -486,18 +496,35 @@ do_setup_full (GstRTSPConnection * conn, const gchar * control,
     }
   }
 
-  if (use_tcp_transport) {
-    transport_string_in = g_strdup_printf (TEST_PROTO_TCP ";unicast");
-  } else {
-    transport_string_in =
-        g_strdup_printf (TEST_PROTO ";unicast;client_port=%d-%d",
+  transport_string_in = g_string_new (TEST_PROTO);
+  switch (lower_transport) {
+    case GST_RTSP_LOWER_TRANS_UDP:
+      transport_string_in =
+          g_string_append (transport_string_in, "/UDP;unicast");
+      break;
+    case GST_RTSP_LOWER_TRANS_UDP_MCAST:
+      transport_string_in =
+          g_string_append (transport_string_in, "/UDP;multicast");
+      break;
+    case GST_RTSP_LOWER_TRANS_TCP:
+      transport_string_in =
+          g_string_append (transport_string_in, "/TCP;unicast");
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+  }
+
+  if (client_ports) {
+    g_string_append_printf (transport_string_in, ";client_port=%d-%d",
         client_ports->min, client_ports->max);
   }
+
   code =
       do_request_full (conn, GST_RTSP_SETUP, control, session_in,
-      transport_string_in, NULL, require, NULL, NULL, NULL, session_out,
+      transport_string_in->str, NULL, require, NULL, NULL, NULL, session_out,
       &transport_string_out, NULL, unsupported);
-  g_free (transport_string_in);
+  g_string_free (transport_string_in, TRUE);
 
   if (transport_string_out) {
     /* create transport */
@@ -519,20 +546,8 @@ do_setup (GstRTSPConnection * conn, const gchar * control,
     const GstRTSPRange * client_ports, gchar ** session,
     GstRTSPTransport ** transport)
 {
-  return do_setup_full (conn, control, FALSE, client_ports, NULL, session,
-      transport, NULL);
-}
-
-/* send a SETUP request and receive response. if *session is not NULL,
- * it is used in the request. otherwise, *session is set to a returned
- * session string that must be freed by the caller. the returned
- * transport must be freed by the caller. */
-static GstRTSPStatusCode
-do_setup_tcp (GstRTSPConnection * conn, const gchar * control,
-    gchar ** session, GstRTSPTransport ** transport)
-{
-  return do_setup_full (conn, control, TRUE, NULL, NULL, session, transport,
-      NULL);
+  return do_setup_full (conn, control, GST_RTSP_LOWER_TRANS_UDP, client_ports,
+      NULL, session, transport, NULL);
 }
 
 /* fixture setup function */
@@ -557,7 +572,7 @@ GST_START_TEST (test_connect)
 {
   GstRTSPConnection *conn;
 
-  start_server ();
+  start_server (FALSE);
 
   /* connect to server */
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
@@ -583,7 +598,7 @@ GST_START_TEST (test_describe)
   const gchar *control_video;
   const gchar *control_audio;
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -654,7 +669,7 @@ GST_START_TEST (test_describe_non_existing_mount_point)
 {
   GstRTSPConnection *conn;
 
-  start_server ();
+  start_server (FALSE);
 
   /* send DESCRIBE request for a non-existing mount point
    * and check that we get a 404 Not Found */
@@ -670,19 +685,20 @@ GST_START_TEST (test_describe_non_existing_mount_point)
 
 GST_END_TEST;
 
-GST_START_TEST (test_setup)
+static void
+do_test_setup (GstRTSPLowerTrans lower_transport)
 {
   GstRTSPConnection *conn;
   GstSDPMessage *sdp_message = NULL;
   const GstSDPMedia *sdp_media;
   const gchar *video_control;
   const gchar *audio_control;
-  GstRTSPRange client_ports;
+  GstRTSPRange client_ports = { 0 };
   gchar *session = NULL;
   GstRTSPTransport *video_transport = NULL;
   GstRTSPTransport *audio_transport = NULL;
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -698,26 +714,28 @@ GST_START_TEST (test_setup)
   get_client_ports (&client_ports);
 
   /* send SETUP request for video */
-  fail_unless (do_setup (conn, video_control, &client_ports, &session,
-          &video_transport) == GST_RTSP_STS_OK);
+  fail_unless (do_setup_full (conn, video_control, lower_transport,
+          &client_ports, NULL, &session, &video_transport,
+          NULL) == GST_RTSP_STS_OK);
   GST_DEBUG ("set up video %s, got session '%s'", video_control, session);
 
   /* check response from SETUP */
   fail_unless (video_transport->trans == GST_RTSP_TRANS_RTP);
   fail_unless (video_transport->profile == GST_RTSP_PROFILE_AVP);
-  fail_unless (video_transport->lower_transport == GST_RTSP_LOWER_TRANS_UDP);
+  fail_unless (video_transport->lower_transport == lower_transport);
   fail_unless (video_transport->mode_play);
   gst_rtsp_transport_free (video_transport);
 
   /* send SETUP request for audio */
-  fail_unless (do_setup (conn, audio_control, &client_ports, &session,
-          &audio_transport) == GST_RTSP_STS_OK);
+  fail_unless (do_setup_full (conn, audio_control, lower_transport,
+          &client_ports, NULL, &session, &audio_transport,
+          NULL) == GST_RTSP_STS_OK);
   GST_DEBUG ("set up audio %s with session '%s'", audio_control, session);
 
   /* check response from SETUP */
   fail_unless (audio_transport->trans == GST_RTSP_TRANS_RTP);
   fail_unless (audio_transport->profile == GST_RTSP_PROFILE_AVP);
-  fail_unless (audio_transport->lower_transport == GST_RTSP_LOWER_TRANS_UDP);
+  fail_unless (audio_transport->lower_transport == lower_transport);
   fail_unless (audio_transport->mode_play);
   gst_rtsp_transport_free (audio_transport);
 
@@ -733,66 +751,23 @@ GST_START_TEST (test_setup)
   iterate ();
 }
 
+GST_START_TEST (test_setup_udp)
+{
+  do_test_setup (GST_RTSP_LOWER_TRANS_UDP);
+}
+
 GST_END_TEST;
 
 GST_START_TEST (test_setup_tcp)
 {
-  GstRTSPConnection *conn;
-  GstSDPMessage *sdp_message = NULL;
-  const GstSDPMedia *sdp_media;
-  const gchar *video_control;
-  const gchar *audio_control;
-  gchar *session = NULL;
-  GstRTSPTransport *video_transport = NULL;
-  GstRTSPTransport *audio_transport = NULL;
+  do_test_setup (GST_RTSP_LOWER_TRANS_TCP);
+}
 
-  start_server ();
+GST_END_TEST;
 
-  conn = connect_to_server (test_port, TEST_MOUNT_POINT);
-
-  sdp_message = do_describe (conn, TEST_MOUNT_POINT);
-
-  /* get control strings from DESCRIBE response */
-  fail_unless (gst_sdp_message_medias_len (sdp_message) == 2);
-  sdp_media = gst_sdp_message_get_media (sdp_message, 0);
-  video_control = gst_sdp_media_get_attribute_val (sdp_media, "control");
-  sdp_media = gst_sdp_message_get_media (sdp_message, 1);
-  audio_control = gst_sdp_media_get_attribute_val (sdp_media, "control");
-
-  /* send SETUP request for video */
-  fail_unless (do_setup_tcp (conn, video_control, &session,
-          &video_transport) == GST_RTSP_STS_OK);
-  GST_DEBUG ("set up video %s, got session '%s'", video_control, session);
-
-  /* check response from SETUP */
-  fail_unless (video_transport->trans == GST_RTSP_TRANS_RTP);
-  fail_unless (video_transport->profile == GST_RTSP_PROFILE_AVP);
-  fail_unless (video_transport->lower_transport == GST_RTSP_LOWER_TRANS_TCP);
-  fail_unless (video_transport->mode_play);
-  gst_rtsp_transport_free (video_transport);
-
-  /* send SETUP request for audio */
-  fail_unless (do_setup_tcp (conn, audio_control, &session,
-          &audio_transport) == GST_RTSP_STS_OK);
-  GST_DEBUG ("set up audio %s with session '%s'", audio_control, session);
-
-  /* check response from SETUP */
-  fail_unless (audio_transport->trans == GST_RTSP_TRANS_RTP);
-  fail_unless (audio_transport->profile == GST_RTSP_PROFILE_AVP);
-  fail_unless (audio_transport->lower_transport == GST_RTSP_LOWER_TRANS_TCP);
-  fail_unless (audio_transport->mode_play);
-  gst_rtsp_transport_free (audio_transport);
-
-  /* send TEARDOWN request and check that we get 200 OK */
-  fail_unless (do_simple_request (conn, GST_RTSP_TEARDOWN,
-          session) == GST_RTSP_STS_OK);
-
-  /* clean up and iterate so the clean-up can finish */
-  g_free (session);
-  gst_sdp_message_free (sdp_message);
-  gst_rtsp_connection_free (conn);
-  stop_server ();
-  iterate ();
+GST_START_TEST (test_setup_udp_mcast)
+{
+  do_test_setup (GST_RTSP_LOWER_TRANS_UDP_MCAST);
 }
 
 GST_END_TEST;
@@ -808,7 +783,7 @@ GST_START_TEST (test_setup_twice)
   gchar *session1 = NULL;
   gchar *session2 = NULL;
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -880,7 +855,7 @@ GST_START_TEST (test_setup_with_require_header)
   gchar *unsupported = NULL;
   GstRTSPTransport *video_transport = NULL;
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -894,17 +869,18 @@ GST_START_TEST (test_setup_with_require_header)
   get_client_ports (&client_ports);
 
   /* send SETUP request for video, with single Require header */
-  fail_unless_equals_int (do_setup_full (conn, video_control, FALSE,
-          &client_ports, "funky-feature", &session, &video_transport,
-          &unsupported), GST_RTSP_STS_OPTION_NOT_SUPPORTED);
+  fail_unless_equals_int (do_setup_full (conn, video_control,
+          GST_RTSP_LOWER_TRANS_UDP, &client_ports, "funky-feature", &session,
+          &video_transport, &unsupported), GST_RTSP_STS_OPTION_NOT_SUPPORTED);
   fail_unless_equals_string (unsupported, "funky-feature");
   g_free (unsupported);
   unsupported = NULL;
 
   /* send SETUP request for video, with multiple Require headers */
-  fail_unless_equals_int (do_setup_full (conn, video_control, FALSE,
-          &client_ports, "funky-feature, foo-bar, superburst", &session,
-          &video_transport, &unsupported), GST_RTSP_STS_OPTION_NOT_SUPPORTED);
+  fail_unless_equals_int (do_setup_full (conn, video_control,
+          GST_RTSP_LOWER_TRANS_UDP, &client_ports,
+          "funky-feature, foo-bar, superburst", &session, &video_transport,
+          &unsupported), GST_RTSP_STS_OPTION_NOT_SUPPORTED);
   fail_unless_equals_string (unsupported, "funky-feature, foo-bar, superburst");
   g_free (unsupported);
   unsupported = NULL;
@@ -941,7 +917,7 @@ GST_START_TEST (test_setup_non_existing_stream)
   GstRTSPConnection *conn;
   GstRTSPRange client_ports;
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -1034,7 +1010,8 @@ done:
 }
 
 static void
-do_test_play (const gchar * range)
+do_test_play_full (const gchar * range, GstRTSPLowerTrans lower_transport,
+    GMutex * lock)
 {
   GstRTSPConnection *conn;
   GstSDPMessage *sdp_message = NULL;
@@ -1062,10 +1039,12 @@ do_test_play (const gchar * range)
   get_client_ports_full (&client_port, &rtp_socket, &rtcp_socket);
 
   /* do SETUP for video and audio */
-  fail_unless (do_setup (conn, video_control, &client_port, &session,
-          &video_transport) == GST_RTSP_STS_OK);
-  fail_unless (do_setup (conn, audio_control, &client_port, &session,
-          &audio_transport) == GST_RTSP_STS_OK);
+  fail_unless (do_setup_full (conn, video_control, lower_transport,
+          &client_port, NULL, &session, &video_transport,
+          NULL) == GST_RTSP_STS_OK);
+  fail_unless (do_setup_full (conn, audio_control, lower_transport,
+          &client_port, NULL, &session, &audio_transport,
+          NULL) == GST_RTSP_STS_OK);
 
   /* send PLAY request and check that we get 200 OK */
   fail_unless (do_request (conn, GST_RTSP_PLAY, NULL, session, NULL, range,
@@ -1074,8 +1053,20 @@ do_test_play (const gchar * range)
     fail_unless_equals_string (range, range_out);
   g_free (range_out);
 
-  receive_rtp (rtp_socket, NULL);
-  receive_rtcp (rtcp_socket, NULL, 0);
+  for (;;) {
+    receive_rtp (rtp_socket, NULL);
+    receive_rtcp (rtcp_socket, NULL, 0);
+
+    if (lock != NULL) {
+      if (g_mutex_trylock (lock) == TRUE) {
+        g_mutex_unlock (lock);
+        break;
+      }
+    } else {
+      break;
+    }
+
+  }
 
   /* send TEARDOWN request and check that we get 200 OK */
   fail_unless (do_simple_request (conn, GST_RTSP_TEARDOWN,
@@ -1096,10 +1087,15 @@ do_test_play (const gchar * range)
   gst_rtsp_connection_free (conn);
 }
 
+static void
+do_test_play (const gchar * range)
+{
+  do_test_play_full (range, GST_RTSP_LOWER_TRANS_UDP, NULL);
+}
 
 GST_START_TEST (test_play)
 {
-  start_server ();
+  start_server (FALSE);
 
   do_test_play (NULL);
 
@@ -1113,7 +1109,7 @@ GST_START_TEST (test_play_without_session)
 {
   GstRTSPConnection *conn;
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -1173,7 +1169,7 @@ GST_START_TEST (test_play_multithreaded)
   gst_rtsp_thread_pool_set_max_threads (pool, 2);
   g_object_unref (pool);
 
-  start_server ();
+  start_server (FALSE);
 
   do_test_play (NULL);
 
@@ -1233,7 +1229,7 @@ GST_START_TEST (test_play_multithreaded_block_in_describe)
   gst_rtsp_mount_points_add_factory (mounts, TEST_MOUNT_POINT "2", factory);
   g_object_unref (mounts);
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT "2");
   iterate ();
@@ -1312,7 +1308,7 @@ GST_START_TEST (test_play_multithreaded_timeout_client)
   g_signal_connect (server, "client-connected",
       G_CALLBACK (session_connected_new_session_cb), new_session_timeout_one);
 
-  start_server ();
+  start_server (FALSE);
 
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
@@ -1329,10 +1325,12 @@ GST_START_TEST (test_play_multithreaded_timeout_client)
   get_client_ports (&client_port);
 
   /* do SETUP for video and audio */
-  fail_unless (do_setup (conn, video_control, &client_port, &session,
-          &video_transport) == GST_RTSP_STS_OK);
-  fail_unless (do_setup (conn, audio_control, &client_port, &session,
-          &audio_transport) == GST_RTSP_STS_OK);
+  fail_unless (do_setup_full (conn, video_control, GST_RTSP_LOWER_TRANS_UDP,
+          &client_port, NULL, &session, &video_transport,
+          NULL) == GST_RTSP_STS_OK);
+  fail_unless (do_setup_full (conn, audio_control, GST_RTSP_LOWER_TRANS_UDP,
+          &client_port, NULL, &session, &audio_transport,
+          NULL) == GST_RTSP_STS_OK);
 
   fail_unless (gst_rtsp_session_pool_get_n_sessions (pool) == 1);
 
@@ -1383,7 +1381,7 @@ GST_START_TEST (test_play_multithreaded_timeout_session)
   g_signal_connect (server, "client-connected",
       G_CALLBACK (session_connected_new_session_cb), new_session_timeout_one);
 
-  start_server ();
+  start_server (FALSE);
 
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
@@ -1459,7 +1457,7 @@ GST_START_TEST (test_play_disconnect)
   g_signal_connect (server, "client-connected",
       G_CALLBACK (session_connected_new_session_cb), new_session_timeout_one);
 
-  start_server ();
+  start_server (FALSE);
 
   conn = connect_to_server (test_port, TEST_MOUNT_POINT);
 
@@ -1529,6 +1527,10 @@ GST_START_TEST (test_play_specific_server_port)
   mounts = gst_rtsp_server_get_mount_points (server);
 
   factory = gst_rtsp_media_factory_new ();
+  /* we have to suspend media after SDP in order to make sure that
+   * we can reconfigure UDP sink with new UDP ports */
+  gst_rtsp_media_factory_set_suspend_mode (factory,
+      GST_RTSP_SUSPEND_MODE_RESET);
   pool = gst_rtsp_address_pool_new ();
   gst_rtsp_address_pool_add_range (pool, GST_RTSP_ADDRESS_POOL_ANY_IPV4,
       GST_RTSP_ADDRESS_POOL_ANY_IPV4, 7770, 7780, 0);
@@ -1616,7 +1618,7 @@ GST_END_TEST;
 
 GST_START_TEST (test_play_smpte_range)
 {
-  start_server ();
+  start_server (FALSE);
 
   do_test_play ("npt=5-");
   do_test_play ("smpte=0:00:00-");
@@ -1630,6 +1632,65 @@ GST_START_TEST (test_play_smpte_range)
 
 GST_END_TEST;
 
+static gpointer
+thread_func (gpointer data)
+{
+  do_test_play_full (NULL, GST_RTSP_LOWER_TRANS_UDP, (GMutex *) data);
+  return NULL;
+}
+
+/* Test adding and removing clients to a 'Shared' media. */
+GST_START_TEST (test_shared)
+{
+  GMutex lock1, lock2, lock3, lock4;
+  GThread *thread1, *thread2, *thread3, *thread4;
+
+  /* Locks for each thread. Each thread will keep reading data as long as the
+   * thread is locked. */
+  g_mutex_init (&lock1);
+  g_mutex_init (&lock2);
+  g_mutex_init (&lock3);
+  g_mutex_init (&lock4);
+
+  start_server (TRUE);
+
+  /* Start the first receiver thread. */
+  g_mutex_lock (&lock1);
+  thread1 = g_thread_new ("thread1", thread_func, &lock1);
+
+  /* Connect and disconnect another client. */
+  g_mutex_lock (&lock2);
+  thread2 = g_thread_new ("thread2", thread_func, &lock2);
+  g_mutex_unlock (&lock2);
+  g_mutex_clear (&lock2);
+  g_thread_join (thread2);
+
+  /* Do it again. */
+  g_mutex_lock (&lock3);
+  thread3 = g_thread_new ("thread3", thread_func, &lock3);
+  g_mutex_unlock (&lock3);
+  g_mutex_clear (&lock3);
+  g_thread_join (thread3);
+
+  /* Disconnect the last client. This will clean up the media. */
+  g_mutex_unlock (&lock1);
+  g_mutex_clear (&lock1);
+  g_thread_join (thread1);
+
+  /* Connect and disconnect another client. This will create and clean up the 
+   * media. */
+  g_mutex_lock (&lock4);
+  thread4 = g_thread_new ("thread4", thread_func, &lock4);
+  g_mutex_unlock (&lock4);
+  g_mutex_clear (&lock4);
+  g_thread_join (thread4);
+
+  stop_server ();
+  iterate ();
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_announce_without_sdp)
 {
   GstRTSPConnection *conn;
@@ -1762,7 +1823,8 @@ GST_START_TEST (test_record_tcp)
   gint i;
 
   mfactory =
-      start_record_server ("( rtppcmadepay name=depay0 ! appsink name=sink )");
+      start_record_server
+      ("( rtppcmadepay name=depay0 ! appsink name=sink async=false )");
 
   g_signal_connect (mfactory, "media-constructed",
       G_CALLBACK (media_constructed_cb), &server_sink);
@@ -1923,8 +1985,9 @@ rtspserver_suite (void)
   tcase_add_test (tc, test_describe);
   tcase_add_test (tc, test_describe_non_existing_mount_point);
   tcase_add_test (tc, test_describe_record_media);
-  tcase_add_test (tc, test_setup);
+  tcase_add_test (tc, test_setup_udp);
   tcase_add_test (tc, test_setup_tcp);
+  tcase_add_test (tc, test_setup_udp_mcast);
   tcase_add_test (tc, test_setup_twice);
   tcase_add_test (tc, test_setup_with_require_header);
   tcase_add_test (tc, test_setup_non_existing_stream);
@@ -1938,6 +2001,7 @@ rtspserver_suite (void)
   tcase_add_test (tc, test_play_disconnect);
   tcase_add_test (tc, test_play_specific_server_port);
   tcase_add_test (tc, test_play_smpte_range);
+  tcase_add_test (tc, test_shared);
   tcase_add_test (tc, test_announce_without_sdp);
   tcase_add_test (tc, test_record_tcp);
   return s;
